import sys
import cherrypy
import traceknack

"""
TraceKnack interface module for CherryPy

You can either derive your controllers' base class from TraceKnackRoot,
below, or just define _cpOnError() yourself.

You need to hook TraceKnackHandler in to your root controller; whatever URL
you map it to, must be passed in to TraceKnackHandler, eg:

    from traceknack.for_cherrypy import TraceKnackRoot, TraceKnackHandler
    
    class MyRoot(TraceKnackRoot):
        debug = TraceKnackHandler("/debug/")
        # ...
        
"""

class TraceKnackRoot(object):    
    def _cpOnError(self):
        try:
            raise
        except:
            cherrypy.response.body = self.debug.knack.handle(sys.exc_info())

        
class TraceKnackHandler(TraceKnackRoot):
    def __init__(self, root):
        self.knack = traceknack.TraceKnack(root)
    
    @cherrypy.expose()
    def Viewer(self, except_id, depth=-1, callables=False, hidden=False):
        depth = int(depth)-1
        if depth<0: depth=None
        return self.knack.render_viewer(int(except_id), depth, bool(callables), bool(hidden))
        
    @cherrypy.expose()
    def Var(self, except_id, depth, path, callables=False, hidden=False):
        depth = int(depth)-1
        if depth<0: depth=None
        return self.knack.get_variables(int(except_id), depth, bool(callables), bool(hidden), path)
        
