import sys
from cgi import parse_qs
from urlparse import urlsplit
import traceknack

"""
WSGI middleware wrapper.

** EXPERIMENTAL **
This has been tested against trivial applications running under the
reference WSGI implementation (http://svn.eby-sarna.com/wsgiref/),
but hasn't been extensively stressed.

Usage: Simply import Middleware and wrap your application with it,
like any other WSGI middleware. Pass in the sub-path you want TraceKnack
to use for debugger callbacks, eg:

    # ...assume myApp has been set up with your application already
    from traceknack.for_wsgi import Middleware as TraceKnack
    myApp = TraceKnack(myApp, '/debug/')

"""

class CaptureFuncCall(object):
    def __init__(self, positional=None, keywords=None):
        if positional is None: positional = []
        if keywords is None: keywords = {}
        self.positional = positional
        self.keyword = keywords
        
    def __call__(self, *pos, **key):
        self.positional.extend(pos)
        self.keyword.update(key)


def unpack_query(environ):
    result = {}
    for key, value in parse_qs(environ.get("QUERY_STRING","")).items():
        result[key] = value[0]
    return result
    

class Middleware(object):
    def __init__(self, application, root):
        self.application = application
        self.root = root
        self.knack = traceknack.TraceKnack(root)
        
    def __call__(self, environ, start_response):
        # is it a debugger call?
        path = environ["SCRIPT_NAME"]+environ["PATH_INFO"]
        if path.startswith(self.root):
            print ">>",path
            command = path.split("/")[-1].title()
            print command
            handler = getattr(self, "Handle"+command)
            if handler:
                return handler(unpack_query(environ), environ, start_response)
            
        # otherwise, pass to application
        try:
            positional = []
            keywords = {}
            result = list(self.application(environ, CaptureFuncCall(positional, keywords)))
            start_response(*positional, **keywords)
            return result
        except:
            start_response("200 OK", [('Content-type', 'text/html')])
            return [self.knack.handle(sys.exc_info())]
            
    def HandleViewer(self, query, environ, start_response):
        except_id = query.get("except_id")
        depth = query.get("depth","-1")
        callables = query.get("callables","0")
        hidden = query.get("hidden","0")
        
        depth = int(depth)-1
        if depth<0: depth=None
        result = self.knack.render_viewer(int(except_id), depth, bool(callables), bool(hidden))
        start_response("200 OK", [('Content-type', 'text/html')])
        return result

    def HandleVar(self, query, environ, start_response):        
        except_id = query.get("except_id")
        depth = query.get("depth")
        path = query.get("path")
        callables = query.get("callables","0")
        hidden = query.get("hidden","0")

        depth = int(depth)-1
        if depth<0: depth=None
        result = self.knack.get_variables(int(except_id), depth, bool(callables), bool(hidden), path)
        start_response("200 OK", [('Content-type', 'text/html')])
        return result
