#!/usr/bin/env python

import tokenize, token, keyword


def escape(item):
    return item.replace('&', '&amp;').replace('<',"&lt;").replace('>', '&gt;')

    
def nameCheck(item, context):
    if keyword.iskeyword(item):
        return "<span class='keyword'>%s</span>" % escape(item)
    userNames = context.get("userNames",{})
    if item in userNames:
        return "<span class='name'>%s</span>" % escape(item)
    if item in __builtins__:
        return "<span class='builtin'>%s</span>" % escape(item)
    return escape(item)
 
    
def nameTracker(item, context):
    userNames = context.setdefault("userNames",set())
    userNames.add(item)
    return "<span class='name'>%s</span>" % escape(item)
 
    
formatters = {
  "COMMENT": lambda x,c: "<span class='comment'>%s</span>" % escape(x),
  "NAME": nameCheck,
  "STRING": lambda x,c: "<span class='string'>%s</span>" % escape(x),
}

specials = {
    'def': 
    {
        'NAME': nameTracker,
    }
}

def generate_html(lines):
    context = {}
    previous = None
    lastY, lastX = None, None
    escaper = lambda item, context: escape(item)
    for (code, text, (Y, X), end, line) in tokenize.generate_tokens(lines):
        toke = token.tok_name[code]
        next = specials.get(previous, formatters).get(toke, escaper)(text, context)
        if lastY!=Y:
            lastX=0
        if lastX!=X:
            next=(" "*(X-lastX))+next
        previous = text.strip()
        lastY, lastX = end
        yield next


def hilight(lines):
    return "".join(list(generate_html(lines)))


class wrapper(object):
    def __init__(self, l):
        self.i=0
        self.l=l
    def read(self):
        try:
            self.i+=1
            return self.l[self.i-1]
        except:
            return ""

    
def hilightList(lines):
    w=wrapper(lines)
    return "".join(list(generate_html(w.read)))

    
def hilightFile(fn):
    return hilight(file(fn,"rU").readline)

__all__ = ["hilightFile", "hilightList", "generate_html"]

if __name__=='__main__':
    import sys
    for fn in sys.argv[1:]:
        print hilightFile(fn)
